@@ -261,35 +261,45 @@ def test_math_module():
         else:
             raise Exception('OverflowError exception expected')
 
-    elif sys.version_info < (3,):
+    else:
 
         try:
             math.log(0)
-        except ValueError, err_math:
-            pass
+        except ValueError, err_math:  # Python 2.6+: as err_math
+            # Python 3 does not make exceptions local variables: they are
+            # restricted to their except block:
+            err_math_args = err_math.args
         else:
             raise Exception('ValueError exception expected')
+
         try:
             umath.log(0)
-        except ValueError, err_ufloat:
-            assert err_math.args == err_ufloat.args
+        except ValueError, err_ufloat:  # Python 2.6+: as err_math
+            assert err_math_args == err_ufloat.args
         else:
             raise Exception('ValueError exception expected')
         try:
             umath.log(uncertainties.ufloat(0, 0))
-        except ValueError, err_ufloat:
-            assert err_math.args == err_ufloat.args
+        except ValueError, err_ufloat:  # Python 2.6+: as err_math
+            assert err_math_args == err_ufloat.args
         else:
             raise Exception('ValueError exception expected')
         try:
             umath.log(uncertainties.ufloat(0, 1))
-        except ValueError, err_ufloat:
-            assert err_math.args == err_ufloat.args
+        except ValueError, err_ufloat:  # Python 2.6+: as err_math
+            assert err_math_args == err_ufloat.args
         else:
             raise Exception('ValueError exception expected')
-        
-    else:  # Python 3+
-        
-        # !!! The tests should be made to work with Python 3 too!
-        pass
+
     
+def test_hypot():
+    '''
+    Special cases where derivatives cannot be calculated:
+    '''
+    x = uncertainties.ufloat(0, 1)
+    y = uncertainties.ufloat(0, 2)
+    # Derivatives that cannot be calculated simply return NaN, with no
+    # exception being raised, normally:
+    result = umath.hypot(x, y)
+    assert test_uncertainties.isnan(result.derivatives[x])
+    assert test_uncertainties.isnan(result.derivatives[y])
