@@ -8,7 +8,7 @@ Examples:
   from umath import sin
   
   # Manipulation of numbers with uncertainties:
-  x = uncertainties.ufloat((3, 0.1))
+  x = uncertainties.ufloat(3, 0.1)
   print sin(x)  # prints 0.141120008...+/-0.098999...
 
   # The umath functions also work on regular Python floats:
@@ -21,7 +21,7 @@ calculator.  Example:
   import uncertainties
   from uncertainties.umath import *  # Imports tan(), etc.
   
-  x = uncertainties.ufloat((3, 0.1))
+  x = uncertainties.ufloat(3, 0.1)
   print tan(x)  # tan() is the uncertainties.umath.tan function
 
 The numbers with uncertainties handled by this module are objects from
@@ -194,10 +194,16 @@ for name in dir(math):
     elif name in num_deriv_funcs:
         # Functions whose derivatives are calculated numerically by
         # this module fall here (isinf, fmod,...):
-        derivatives = None  # Means: numerical calculation required
+        derivatives = []  # Means: numerical calculation required
     else:
         continue  # 'name' not wrapped by this module (__doc__, e, etc.)
 
+    # Errors during the calculation of the derivatives are converted
+    # to a NaN result: it is assumed that a mathematical calculation
+    # that cannot be calculated indicates a non-defined derivative
+    # (the derivatives in fixed_derivatives must be written this way):
+    derivatives = map(uncertainties.nan_if_exception, derivatives)
+    
     func = getattr(math, name)
     
     setattr(this_module, name,
